/*--------------------------------------------------------------------------*\
 |                                                                          |
 |  Copyright (C) 2003-2010                                                 |
 |                                                                          |
 |         , __                 , __                                        |
 |        /|/  \               /|/  \                                       |
 |         | __/ _   ,_         | __/ _   ,_                                | 
 |         |   \|/  /  |  |   | |   \|/  /  |  |   |                        |
 |         |(__/|__/   |_/ \_/|/|(__/|__/   |_/ \_/|/                       |
 |                           /|                   /|                        |
 |                           \|                   \|                        |
 |                                                                          |
 |      Enrico Bertolazzi                                                   |
 |      Dipartimento di Ingegneria Meccanica e Strutturale                  |
 |      Universita` degli Studi di Trento                                   |
 |      Via Mesiano 77, I-38050 Trento, Italy                               |
 |      email: enrico.bertolazzi@unitn.it                                   |
 |                                                                          |
\*--------------------------------------------------------------------------*/

#include "Malloc.hh"

#include <sstream>
#include <stdexcept>

using namespace std ;

#ifndef ASSERT
  #define ASSERT(COND,MSG) \
    if ( !(COND) ) { ostringstream ost ; ost << MSG << '\n' ; throw runtime_error(ost.str()) ; }
#endif

/*
//   __  __       _ _            
//  |  \/  | __ _| | | ___   ___ 
//  | |\/| |/ _` | | |/ _ \ / __|
//  | |  | | (_| | | | (_) | (__ 
//  |_|  |_|\__,_|_|_|\___/ \___|
*/

template <typename T>
Malloc<T>::Malloc(string const & _name)
: theName(_name)
, numTotValues(0)
, numTotReserved(0)
, numAllocated(0)
, pMalloc(0)
{}
    
template <typename T>
Malloc<T>::~Malloc()
{ this -> free() ; }

template <typename T>
void
Malloc<T>::allocate( indexType n ) {
  if ( n <= numTotReserved ) { // nothing to allocate
    numTotValues = n ;
    numAllocated = 0 ;
  } else {
    if ( pMalloc != 0 ) delete [] pMalloc ;
    numTotValues   = n ;
    numTotReserved = n + n / 10 ; // 10% more values
    #ifdef DEBUG
    indexType numBytes = numTotReserved*sizeof(valueType) ;
    cerr << "\n\nAllocating for " << theName << "  "
         << numBytes << " bytes [ "
         << int(numBytes/10.24)/100 << " Kb, " 
         << int(numBytes/(1024*10.24))/100 << " Mb ]\n\n\n" ; 
    #endif
    numAllocated   = 0 ;
    try {
      pMalloc = new T[numTotReserved] ;
    } catch (...) {
      ASSERT(false, "Malloc allocation failed for " << theName << ": Malloc exausted\n" ) ;
    }
  }
}

template <typename T>
void
Malloc<T>::free(void) {
  if ( pMalloc != 0 ) {
    #ifdef DEBUG
    indexType numBytes = numTotReserved*sizeof(valueType) ;
    cerr << "\nFreeing for " << theName << "  "
         << numBytes << " bytes [ "
         << int(numBytes/10.24)/100 << " Kb, " 
         << int(numBytes/(1024*10.24))/100 << " Mb ]\n\n\n" ; 
    #endif
    delete [] pMalloc ;
    pMalloc        = 0 ;
    numTotValues   = 0 ;
    numTotReserved = 0 ;
    numAllocated   = 0 ;
  }
}

template <typename T>
T *
Malloc<T>::operator () ( indexType sz ) {
  indexType offs = numAllocated ;
  numAllocated += sz ;
  ASSERT( numAllocated <= numTotValues,
          "\nMalloc<" << theName << ">::operator () (" << sz << ") -- Malloc EXAUSTED\n" << 
          "request = " << numAllocated << " > " << numTotValues << " = available\n" ) ;
  return pMalloc + offs ;
}

/*
//   ___           _              _   _       _       
//  |_ _|_ __  ___| |_ __ _ _ __ | |_(_) __ _| |_ ___ 
//   | || '_ \/ __| __/ _` | '_ \| __| |/ _` | __/ _ \
//   | || | | \__ \ || (_| | | | | |_| | (_| | ||  __/
//  |___|_| |_|___/\__\__,_|_| |_|\__|_|\__,_|\__\___|
*/

template class Malloc<int> ;
template class Malloc<unsigned> ;
template class Malloc<float> ;
template class Malloc<double> ;
